@@ -15,13 +15,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <QCryptographicHash>
 #include <QIODevice>
 #include <QDBusMessage>
-#include <QDBusConnection>
 #include <QCoreApplication>
 #include <QTextStream>
 #include <QFile>
@@ -34,6 +33,8 @@
 #include "interfaces/dbushelpers.h"
 #include "kdeconnect-version.h"
 
+#include <dbushelper.h>
+
 int main(int argc, char** argv)
 {
     QCoreApplication app(argc, argv);
@@ -41,7 +42,7 @@ int main(int argc, char** argv)
                      QStringLiteral("kdeconnect-cli"),
                      QStringLiteral(KDECONNECT_VERSION_STRING),
                      i18n("KDE Connect CLI tool"),
-                     KAboutLicense::GPL, 
+                     KAboutLicense::GPL,
                      i18n("(C) 2015 Aleix Pol Gonzalez"));
     KAboutData::setApplicationData(about);
 
@@ -60,6 +61,7 @@ int main(int argc, char** argv)
     parser.addOption(QCommandLineOption(QStringLiteral("ping"), i18n("Sends a ping to said device")));
     parser.addOption(QCommandLineOption(QStringLiteral("ping-msg"), i18n("Same as ping but you can set the message to display"), i18n("message")));
     parser.addOption(QCommandLineOption(QStringLiteral("share"), i18n("Share a file to a said device"), QStringLiteral("path")));
+    parser.addOption(QCommandLineOption(QStringLiteral("share-text"), i18n("Share text to a said device"), QStringLiteral("text")));
     parser.addOption(QCommandLineOption(QStringLiteral("list-notifications"), i18n("Display the notifications on a said device")));
     parser.addOption(QCommandLineOption(QStringLiteral("lock"), i18n("Lock the specified device")));
     parser.addOption(QCommandLineOption(QStringLiteral("send-sms"), i18n("Sends an SMS. Requires destination"), i18n("message")));
@@ -71,13 +73,20 @@ int main(int argc, char** argv)
     parser.addOption(QCommandLineOption(QStringLiteral("execute-command"), i18n("Executes a remote command by id"), QStringLiteral("id")));
     parser.addOption(QCommandLineOption(QStringList{QStringLiteral("k"), QStringLiteral("send-keys")}, i18n("Sends keys to a said device"), QStringLiteral("key")));
     parser.addOption(QCommandLineOption(QStringLiteral("my-id"), i18n("Display this device's id and exit")));
+    parser.addOption(QCommandLineOption(QStringLiteral("photo"), i18n("Open the connected device's camera and transfer the photo"), QStringLiteral("path")));
+
+    //Hidden because it's an implementation detail
+    QCommandLineOption deviceAutocomplete(QStringLiteral("shell-device-autocompletion"));
+    deviceAutocomplete.setFlags(QCommandLineOption::HiddenFromHelp);
+    deviceAutocomplete.setDescription(QStringLiteral("Outputs all available devices id's with their name and paired status")); //Not visible, so no translation needed
+    deviceAutocomplete.setValueName(QStringLiteral("shell"));
+    parser.addOption(deviceAutocomplete);
     about.setupCommandLine(&parser);
 
-    parser.addHelpOption();
     parser.process(app);
     about.processCommandLine(&parser);
 
-    const QString id = "kdeconnect-cli-"+QString::number(QCoreApplication::applicationPid());
+    const QString id = QStringLiteral("kdeconnect-cli-") + QString::number(QCoreApplication::applicationPid());
     DaemonDbusInterface iface;
 
     if (parser.isSet(QStringLiteral("my-id"))) {
@@ -122,15 +131,45 @@ int main(int argc, char** argv)
             }
         }
         if (displayCount) {
-            QTextStream(stdout) << i18np("1 device found", "%1 devices found", devices.size()) << endl;
+            QTextStream(stderr) << i18np("1 device found", "%1 devices found", devices.size()) << endl;
         } else if (devices.isEmpty()) {
             QTextStream(stderr) << i18n("No devices found") << endl;
         }
 
         blockOnReply(iface.releaseDiscoveryMode(id));
+    } else if (parser.isSet(QStringLiteral("shell-device-autocompletion"))) {
+        //Outputs a list of reachable devices in zsh autocomplete format, with the name as description
+        const QStringList devices = blockOnReply<QStringList>(iface.devices(true, false));
+        for (const QString &id : devices) {
+            DeviceDbusInterface deviceIface(id);
+            QString statusInfo;
+            const bool isTrusted = deviceIface.isTrusted();
+            if (isTrusted) {
+                statusInfo = i18n("(paired)");
+            } else {
+                statusInfo = i18n("(unpaired)");
+            }
+
+            //Description: "device name (paired/unpaired)"
+            QString description = deviceIface.name() + QLatin1Char(' ') + statusInfo;
+            //Replace characters
+            description.replace(QLatin1Char('\\'), QStringLiteral("\\\\"));
+            description.replace(QLatin1Char('['),  QStringLiteral("\\["));
+            description.replace(QLatin1Char(']'),  QStringLiteral("\\]"));
+            description.replace(QLatin1Char('\''), QStringLiteral("\\'"));
+            description.replace(QLatin1Char('\"'), QStringLiteral("\\\""));
+            description.replace(QLatin1Char('\n'), QLatin1Char(' '));
+            description.remove(QLatin1Char('\0'));
+
+            //Output id and description
+            QTextStream(stdout) << id << '[' << description << ']' << endl;
+        }
+
+        //Exit with 1 if we didn't find a device
+        return int(devices.isEmpty());
     } else if(parser.isSet(QStringLiteral("refresh"))) {
         QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kdeconnect"), QStringLiteral("/modules/kdeconnect"), QStringLiteral("org.kde.kdeconnect.daemon"), QStringLiteral("forceOnNetworkChange"));
-        blockOnReply(QDBusConnection::sessionBus().asyncCall(msg));
+        blockOnReply(DBusHelper::sessionBus().asyncCall(msg));
     } else {
 
         QString device = parser.value(QStringLiteral("device"));
@@ -146,28 +185,38 @@ int main(int argc, char** argv)
             parser.showHelp(1);
         }
 
+        if (!blockOnReply<QStringList>(iface.devices(false, false)).contains(device)) {
+            QTextStream(stderr) << "Couldn't find device with id \"" << device << "\". To specify a device by name use -n <devicename>" << endl;
+            return 1;
+        }
+
         if (parser.isSet(QStringLiteral("share"))) {
-            QList<QUrl> urls;
-            QUrl url = QUrl::fromUserInput(parser.value(QStringLiteral("share")));
-            urls.append(url);
+            QStringList urls;
 
-            //In case there are more arguments, check if they are files and then send them
-            for (const QString& file : parser.positionalArguments()) {
-                QUrl url = QUrl::fromUserInput(file, QDir::currentPath());
-                if (!url.isLocalFile()) {
-                    QTextStream(stderr) << i18n("Can't find the file: %1", url.toString()) << endl;
-                    return 1;
-                } else {
-                    urls.append(url);
-                }
+            QUrl url = QUrl::fromUserInput(parser.value(QStringLiteral("share")), QDir::currentPath());
+            urls.append(url.toString());
+
+            // Check for more arguments
+            const auto args = parser.positionalArguments();
+            for (const QString& input : args) {
+                QUrl url = QUrl::fromUserInput(input, QDir::currentPath());
+                urls.append(url.toString());
             }
 
-            for (const QUrl& url : urls) {
-                QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kdeconnect"), "/modules/kdeconnect/devices/"+device+"/share", QStringLiteral("org.kde.kdeconnect.device.share"), QStringLiteral("shareUrl"));
-                msg.setArguments(QVariantList() << url.toString());
-                blockOnReply(QDBusConnection::sessionBus().asyncCall(msg));
-                QTextStream(stdout) << i18n("Sent %1", url.toString()) << endl;
+            QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kdeconnect"), QStringLiteral("/modules/kdeconnect/devices/") + device + QStringLiteral("/share"),
+                                                              QStringLiteral("org.kde.kdeconnect.device.share"), QStringLiteral("shareUrls"));
+
+            msg.setArguments(QVariantList() << QVariant(urls));
+            blockOnReply(DBusHelper::sessionBus().asyncCall(msg));
+
+            for (const QString& url : qAsConst(urls)) {
+                QTextStream(stdout) << i18n("Shared %1", url) << endl;
             }
+        } else if (parser.isSet(QStringLiteral("share-text"))) {
+            QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kdeconnect"), QStringLiteral("/modules/kdeconnect/devices/") + device + QStringLiteral("/share"), QStringLiteral("org.kde.kdeconnect.device.share"), QStringLiteral("shareText"));
+            msg.setArguments(QVariantList() << parser.value(QStringLiteral("share-text")));
+            blockOnReply(DBusHelper::sessionBus().asyncCall(msg));
+            QTextStream(stdout) << i18n("Shared text: %1", parser.value(QStringLiteral("share-text"))) << endl;
         } else if(parser.isSet(QStringLiteral("pair"))) {
             DeviceDbusInterface dev(device);
             if (!dev.isReachable()) {
@@ -176,7 +225,7 @@ int main(int argc, char** argv)
                 QTextStream(stderr) << i18n("waiting for device...") << endl;
                 blockOnReply(iface.acquireDiscoveryMode(id));
 
-                QObject::connect(&iface, &DaemonDbusInterface::deviceAdded, [&](const QString& deviceAddedId) {
+                QObject::connect(&iface, &DaemonDbusInterface::deviceAdded, &iface, [&](const QString& deviceAddedId) {
                     if (device == deviceAddedId) {
                         wait.quit();
                     }
@@ -197,50 +246,57 @@ int main(int argc, char** argv)
             blockOnReply(iface.releaseDiscoveryMode(id));
         } else if(parser.isSet(QStringLiteral("unpair"))) {
             DeviceDbusInterface dev(device);
-            if (!dev.isReachable()) {
-                QTextStream(stderr) << i18n("Device does not exist") << endl;
-            } else if(!dev.isTrusted()) {
+            if (!dev.isTrusted()) {
                 QTextStream(stderr) << i18n("Already not paired") << endl;
             } else {
                 QTextStream(stderr) << i18n("Unpaired") << endl;
                 blockOnReply(dev.unpair());
             }
         } else if(parser.isSet(QStringLiteral("ping")) || parser.isSet(QStringLiteral("ping-msg"))) {
-            QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kdeconnect"), "/modules/kdeconnect/devices/"+device+"/ping", QStringLiteral("org.kde.kdeconnect.device.ping"), QStringLiteral("sendPing"));
+            QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kdeconnect"), QStringLiteral("/modules/kdeconnect/devices/") + device + QStringLiteral("/ping"), QStringLiteral("org.kde.kdeconnect.device.ping"), QStringLiteral("sendPing"));
             if (parser.isSet(QStringLiteral("ping-msg"))) {
                 QString message = parser.value(QStringLiteral("ping-msg"));
                 msg.setArguments(QVariantList() << message);
             }
-            blockOnReply(QDBusConnection::sessionBus().asyncCall(msg));
+            blockOnReply(DBusHelper::sessionBus().asyncCall(msg));
         } else if(parser.isSet(QStringLiteral("send-sms"))) {
             if (parser.isSet(QStringLiteral("destination"))) {
-                QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kdeconnect"), "/modules/kdeconnect/devices/"+device+"/telephony", QStringLiteral("org.kde.kdeconnect.device.telephony"), QStringLiteral("sendSms"));
-                msg.setArguments({ parser.value("destination"), parser.value("send-sms") });
-                blockOnReply(QDBusConnection::sessionBus().asyncCall(msg));
+                QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kdeconnect"), QStringLiteral("/modules/kdeconnect/devices/") + device + QStringLiteral("/sms"), QStringLiteral("org.kde.kdeconnect.device.sms"), QStringLiteral("sendSms"));
+                msg.setArguments({ parser.value(QStringLiteral("destination")), parser.value(QStringLiteral("send-sms"))});
+                blockOnReply(DBusHelper::sessionBus().asyncCall(msg));
             } else {
                 QTextStream(stderr) << i18n("error: should specify the SMS's recipient by passing --destination <phone number>");
                 return 1;
             }
         } else if(parser.isSet(QStringLiteral("ring"))) {
-            QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kdeconnect"), "/modules/kdeconnect/devices/"+device+"/findmyphone", QStringLiteral("org.kde.kdeconnect.device.findmyphone"), QStringLiteral("ring"));
-            blockOnReply(QDBusConnection::sessionBus().asyncCall(msg));
-        } else if(parser.isSet("send-keys")) {
-            QString seq = parser.value("send-keys");
-            QDBusMessage msg = QDBusMessage::createMethodCall("org.kde.kdeconnect", "/modules/kdeconnect/devices/"+device+"/remotekeyboard", "org.kde.kdeconnect.device.remotekeyboard", "sendKeyPress");
+            QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kdeconnect"), QStringLiteral("/modules/kdeconnect/devices/") + device + QStringLiteral("/findmyphone"), QStringLiteral("org.kde.kdeconnect.device.findmyphone"), QStringLiteral("ring"));
+            blockOnReply(DBusHelper::sessionBus().asyncCall(msg));
+        } else if(parser.isSet(QStringLiteral("photo"))) {
+            const QString fileName = parser.value(QStringLiteral("photo"));
+            if (!fileName.isEmpty()) {
+                QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kdeconnect"), QStringLiteral("/modules/kdeconnect/devices/") + device + QStringLiteral("/photo"), QStringLiteral("org.kde.kdeconnect.device.photo"), QStringLiteral("requestPhoto"));
+                msg.setArguments({fileName});
+                blockOnReply(DBusHelper::sessionBus().asyncCall(msg));
+            } else {
+                QTextStream(stderr) << i18n("Please specify a filename for the photo") << endl;
+            }
+        } else if(parser.isSet(QStringLiteral("send-keys"))) {
+            QString seq = parser.value(QStringLiteral("send-keys"));
+            QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kdeconnect"), QStringLiteral("/modules/kdeconnect/devices/") + device + QStringLiteral("/remotekeyboard"), QStringLiteral("org.kde.kdeconnect.device.remotekeyboard"), QStringLiteral("sendKeyPress"));
             if (seq.trimmed() == QLatin1String("-")) {
-                // from file
+                // from stdin
                 QFile in;
                 if(in.open(stdin,QIODevice::ReadOnly | QIODevice::Unbuffered)) {
                     while (!in.atEnd()) {
                         QByteArray line = in.readLine();  // sanitize to ASCII-codes > 31?
-                        msg.setArguments({QString(line), -1, false, false, false});
-                        blockOnReply(QDBusConnection::sessionBus().asyncCall(msg));
+                        msg.setArguments({QString::fromLatin1(line), -1, false, false, false});
+                        blockOnReply(DBusHelper::sessionBus().asyncCall(msg));
                     }
                     in.close();
                 }
             } else {
                 msg.setArguments({seq, -1, false, false, false});
-                blockOnReply(QDBusConnection::sessionBus().asyncCall(msg));
+                blockOnReply(DBusHelper::sessionBus().asyncCall(msg));
             }
         } else if(parser.isSet(QStringLiteral("list-notifications"))) {
             NotificationsModel notifications;
