@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <QApplication>
@@ -25,8 +25,9 @@
 #include <QCommandLineOption>
 #include <QCommandLineParser>
 #include <QDBusMessage>
-#include <QDBusConnection>
 #include <QSessionManager>
+#include <QStandardPaths>
+#include <QIcon>
 
 #include <KAboutData>
 #include <KDBusService>
@@ -34,29 +35,37 @@
 #include <KLocalizedString>
 #include <KIO/AccessManager>
 
+#include <dbushelper.h>
+
 #include "core/daemon.h"
 #include "core/device.h"
 #include "core/backends/pairinghandler.h"
 #include "kdeconnect-version.h"
 
+Q_DECLARE_LOGGING_CATEGORY(KDECONNECT_DAEMON)
+Q_LOGGING_CATEGORY(KDECONNECT_DAEMON, "kdeconnect.daemon")
+
 class DesktopDaemon : public Daemon
 {
     Q_OBJECT
     Q_CLASSINFO("D-Bus Interface", "org.kde.kdeconnect.daemon")
 public:
-    DesktopDaemon(QObject* parent = Q_NULLPTR)
+    DesktopDaemon(QObject* parent = nullptr)
         : Daemon(parent)
-        , m_nam(Q_NULLPTR)
-    {}
+        , m_nam(nullptr)
+    {
+        qApp->setWindowIcon(QIcon(QStandardPaths::locate(QStandardPaths::AppLocalDataLocation, QStringLiteral("icons/hicolor/scalable/apps/kdeconnect.svgz"))));
+    }
 
     void askPairingConfirmation(Device* device) override
     {
-        KNotification* notification = new KNotification(QStringLiteral("pairingRequest"));
+        KNotification* notification = new KNotification(QStringLiteral("pairingRequest"), KNotification::NotificationFlag::Persistent);
+        QTimer::singleShot(PairingHandler::pairingTimeoutMsec(), notification, &KNotification::close);
         notification->setIconName(QStringLiteral("dialog-information"));
         notification->setComponentName(QStringLiteral("kdeconnect"));
+        notification->setTitle(QStringLiteral("KDE Connect"));
         notification->setText(i18n("Pairing request from %1", device->name().toHtmlEscaped()));
         notification->setActions(QStringList() << i18n("Accept") << i18n("Reject"));
-//         notification->setTimeout(PairingHandler::pairingTimeoutMsec());
         connect(notification, &KNotification::action1Activated, device, &Device::acceptPairing);
         connect(notification, &KNotification::action2Activated, device, &Device::rejectPairing);
         notification->sendEvent();
@@ -64,6 +73,7 @@ public:
 
     void reportError(const QString & title, const QString & description) override
     {
+        qCWarning(KDECONNECT_DAEMON) << title << ":" << description;
         KNotification::event(KNotification::Error, title, description);
     }
 
@@ -75,6 +85,11 @@ public:
         return m_nam;
     }
 
+    KJobTrackerInterface* jobTracker() override
+    {
+        return KIO::getJobTracker();
+    }
+
     Q_SCRIPTABLE void sendSimpleNotification(const QString &eventId, const QString &title, const QString &text, const QString &iconName) override
     {
         KNotification* notification = new KNotification(eventId); //KNotification::Persistent
@@ -85,6 +100,9 @@ public:
         notification->sendEvent();
     }
 
+    void quit() override {
+        QApplication::quit();
+    }
 
 private:
     QNetworkAccessManager* m_nam;
@@ -117,11 +135,11 @@ static void detectPlatform(int argc, char **argv)
 
 int main(int argc, char* argv[])
 {
-   detectPlatform(argc, argv);
+    detectPlatform(argc, argv);
 
     QApplication app(argc, argv);
     KAboutData aboutData(
-        QStringLiteral("org.kde.kdeconnect.daemon"),
+        QStringLiteral("kdeconnect.daemon"),
         i18n("KDE Connect Daemon"),
         QStringLiteral(KDECONNECT_VERSION_STRING),
         i18n("KDE Connect Daemon"),
@@ -130,6 +148,10 @@ int main(int argc, char* argv[])
     KAboutData::setApplicationData(aboutData);
     app.setQuitOnLastWindowClosed(false);
 
+#ifdef USE_PRIVATE_DBUS
+    DBusHelper::launchDBusDaemon();
+#endif
+
     QCommandLineParser parser;
     QCommandLineOption replaceOption({QStringLiteral("replace")}, i18n("Replace an existing instance"));
     parser.addOption(replaceOption);
@@ -138,17 +160,16 @@ int main(int argc, char* argv[])
     parser.process(app);
     aboutData.processCommandLine(&parser);
     if (parser.isSet(replaceOption)) {
-        auto message = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kdeconnectd"),
+        auto message = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kdeconnect"),
                                                     QStringLiteral("/MainApplication"),
                                                     QStringLiteral("org.qtproject.Qt.QCoreApplication"),
                                                     QStringLiteral("quit"));
-        QDBusConnection::sessionBus().call(message); //deliberately block until it's done, so we register the name after the app quits
+        DBusHelper::sessionBus().call(message); //deliberately block until it's done, so we register the name after the app quits
     }
 
     KDBusService dbusService(KDBusService::Unique);
 
-    Daemon* daemon = new DesktopDaemon;
-    QObject::connect(daemon, SIGNAL(destroyed(QObject*)), &app, SLOT(quit()));
+    DesktopDaemon daemon;
 
     // kdeconnectd is autostarted, so disable session management to speed up startup
     auto disableSessionManagement = [](QSessionManager &sm) {
