@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "clipboardplugin.h"
@@ -24,7 +24,7 @@
 
 #include <KPluginFactory>
 
-K_PLUGIN_FACTORY_WITH_JSON( KdeConnectPluginFactory, "kdeconnect_clipboard.json", registerPlugin< ClipboardPlugin >(); )
+K_PLUGIN_CLASS_WITH_JSON(ClipboardPlugin, "kdeconnect_clipboard.json")
 
 Q_LOGGING_CATEGORY(KDECONNECT_PLUGIN_CLIPBOARD, "kdeconnect.plugin.clipboard")
 
@@ -35,17 +35,44 @@ ClipboardPlugin::ClipboardPlugin(QObject* parent, const QVariantList& args)
             this, &ClipboardPlugin::propagateClipboard);
 }
 
-void ClipboardPlugin::propagateClipboard(const QString& content)
+void ClipboardPlugin::connected()
 {
-    NetworkPackage np(PACKAGE_TYPE_CLIPBOARD, {{"content", content}});
-    sendPackage(np);
+    sendConnectPacket();
 }
 
-bool ClipboardPlugin::receivePackage(const NetworkPackage& np)
+void ClipboardPlugin::propagateClipboard(const QString& content)
+{
+    NetworkPacket np(PACKET_TYPE_CLIPBOARD, {{QStringLiteral("content"), content}});
+    sendPacket(np);
+}
+
+void ClipboardPlugin::sendConnectPacket()
+{
+    NetworkPacket np(PACKET_TYPE_CLIPBOARD_CONNECT, {
+        {QStringLiteral("content"), ClipboardListener::instance()->currentContent()},
+        {QStringLiteral("timestamp"), ClipboardListener::instance()->updateTimestamp()}
+    });
+    sendPacket(np);
+}
+
+
+bool ClipboardPlugin::receivePacket(const NetworkPacket& np)
 {
     QString content = np.get<QString>(QStringLiteral("content"));
-    ClipboardListener::instance()->setText(content);
-    return true;
+    if (np.type() == PACKET_TYPE_CLIPBOARD) {
+        ClipboardListener::instance()->setText(content);
+        return true;
+    } else if (np.type() == PACKET_TYPE_CLIPBOARD_CONNECT) {
+        qint64 packetTime = np.get<qint64>(QStringLiteral("timestamp"));
+        // If the packetTime is 0, it means the timestamp is unknown (so do nothing).
+        if (packetTime == 0 || packetTime < ClipboardListener::instance()->updateTimestamp()) {
+            return false;
+        }
+
+        ClipboardListener::instance()->setText(content);
+        return true;
+    }
+    return false;
 }
 
 #include "clipboardplugin.moc"
