@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "shareplugin.h"
@@ -26,22 +26,25 @@
 #include <QDir>
 #include <QDesktopServices>
 #include <QDBusConnection>
-#include <QDebug>
 #include <QTemporaryFile>
+#include <QDateTime>
 
 #include <KLocalizedString>
 #include <KJobTrackerInterface>
 #include <KPluginFactory>
 #include <KIO/MkpathJob>
+#include <KMimeTypeTrader>
 
 #include "core/filetransferjob.h"
+#include "core/daemon.h"
 
-K_PLUGIN_FACTORY_WITH_JSON( KdeConnectPluginFactory, "kdeconnect_share.json", registerPlugin< SharePlugin >(); )
+K_PLUGIN_CLASS_WITH_JSON(SharePlugin, "kdeconnect_share.json")
 
 Q_LOGGING_CATEGORY(KDECONNECT_PLUGIN_SHARE, "kdeconnect.plugin.share")
 
 SharePlugin::SharePlugin(QObject* parent, const QVariantList& args)
     : KdeConnectPlugin(parent, args)
+    , m_compositeJob()
 {
 }
 
@@ -63,27 +66,47 @@ QUrl SharePlugin::destinationDir() const
     return dir;
 }
 
+QUrl SharePlugin::getFileDestination(const QString filename) const
+{
+    const QUrl dir = destinationDir().adjusted(QUrl::StripTrailingSlash);
+    QUrl destination(dir);
+    destination.setPath(dir.path() + QStringLiteral("/") + filename, QUrl::DecodedMode);
+    if (destination.isLocalFile() && QFile::exists(destination.toLocalFile())) {
+        destination.setPath(dir.path() + QStringLiteral("/") + KIO::suggestName(dir, filename), QUrl::DecodedMode);
+    }
+    return destination;
+}
+
 static QString cleanFilename(const QString &filename)
 {
     int idx = filename.lastIndexOf(QLatin1Char('/'));
     return idx>=0 ? filename.mid(idx + 1) : filename;
 }
 
-bool SharePlugin::receivePackage(const NetworkPackage& np)
+void SharePlugin::setDateModified(const QUrl& destination, const qint64 timestamp)
+{
+    QFile receivedFile(destination.toLocalFile());
+    if (!receivedFile.exists() || !receivedFile.open(QIODevice::ReadWrite | QIODevice::Text)) {
+        return;
+    }
+    receivedFile.setFileTime(QDateTime::fromMSecsSinceEpoch(timestamp), QFileDevice::FileTime(QFileDevice::FileModificationTime));
+}
+
+bool SharePlugin::receivePacket(const NetworkPacket& np)
 {
 /*
     //TODO: Write a test like this
-    if (np.type() == PACKAGE_TYPE_PING) {
+    if (np.type() == PACKET_TYPE_PING) {
 
         qCDebug(KDECONNECT_PLUGIN_SHARE) << "sending file" << (QDesktopServices::storageLocation(QDesktopServices::HomeLocation) + "/.bashrc");
 
-        NetworkPackage out(PACKAGE_TYPE_SHARE_REQUEST);
+        NetworkPacket out(PACKET_TYPE_SHARE_REQUEST);
         out.set("filename", mDestinationDir + "itworks.txt");
         AutoClosingQFile* file = new AutoClosingQFile(QDesktopServices::storageLocation(QDesktopServices::HomeLocation) + "/.bashrc"); //Test file to transfer
 
         out.setPayload(file, file->size());
 
-        device()->sendPackage(out);
+        device()->sendPacket(out);
 
         return true;
 
@@ -92,31 +115,47 @@ bool SharePlugin::receivePackage(const NetworkPackage& np)
 
     qCDebug(KDECONNECT_PLUGIN_SHARE) << "File transfer";
 
-    if (np.hasPayload()) {
-        const QString filename = cleanFilename(np.get<QString>(QStringLiteral("filename"), QString::number(QDateTime::currentMSecsSinceEpoch())));
-        const QUrl dir = destinationDir().adjusted(QUrl::StripTrailingSlash);
-        QUrl destination(dir);
-        destination.setPath(dir.path() + '/' + filename, QUrl::DecodedMode);
-        if (destination.isLocalFile() && QFile::exists(destination.toLocalFile())) {
-            destination.setPath(dir.path() + '/' + KIO::suggestName(dir, filename), QUrl::DecodedMode);
-        }
+    if (np.hasPayload() || np.has(QStringLiteral("filename"))) {
 //         qCDebug(KDECONNECT_PLUGIN_SHARE) << "receiving file" << filename << "in" << dir << "into" << destination;
+        const QString filename = cleanFilename(np.get<QString>(QStringLiteral("filename"), QString::number(QDateTime::currentMSecsSinceEpoch())));
+        QUrl destination = getFileDestination(filename);
 
-        FileTransferJob* job = np.createPayloadTransferJob(destination);
-        job->setOriginName(device()->name() + ": " + filename);
-        connect(job, &KJob::result, this, &SharePlugin::finished);
-        KIO::getJobTracker()->registerJob(job);
-        job->start();
+        if (np.hasPayload()) {
+            qint64 dateModified = np.get<qint64>(QStringLiteral("lastModified"), QDateTime::currentMSecsSinceEpoch());
+
+            if (!m_compositeJob) {
+                m_compositeJob = new CompositeFileTransferJob(device()->id());
+                KIO::getJobTracker()->registerJob(m_compositeJob);
+            }
+
+            FileTransferJob* job = np.createPayloadTransferJob(destination);
+            job->setOriginName(device()->name() + QStringLiteral(": ") + filename);
+            connect(job, &KJob::result, this, [this, dateModified] (KJob* job) -> void { finished(job, dateModified); });
+            m_compositeJob->addSubjob(job);
+
+            if (!m_compositeJob->isRunning()) {
+                m_compositeJob->start();
+            }
+        } else {
+            QFile file(destination.toLocalFile());
+            file.open(QIODevice::WriteOnly);
+            file.close();
+        }
     } else if (np.has(QStringLiteral("text"))) {
         QString text = np.get<QString>(QStringLiteral("text"));
-        if (!QStandardPaths::findExecutable(QStringLiteral("kate")).isEmpty()) {
+
+        KService::Ptr service = KMimeTypeTrader::self()->preferredService(QStringLiteral("text/plain"));
+        const QString defaultApp = service ? service->desktopEntryName() : QString();
+
+        if (defaultApp == QLatin1String("org.kde.kate") || defaultApp == QLatin1String("org.kde.kwrite")) {
             QProcess* proc = new QProcess();
             connect(proc, SIGNAL(finished(int)), proc, SLOT(deleteLater()));
-            proc->start(QStringLiteral("kate"), QStringList(QStringLiteral("--stdin")));
+            proc->start(defaultApp.section(QStringLiteral("."), 2,2), QStringList(QStringLiteral("--stdin")));
             proc->write(text.toUtf8());
             proc->closeWriteChannel();
         } else {
             QTemporaryFile tmpFile;
+            tmpFile.setFileTemplate(QStringLiteral("kdeconnect-XXXXXX.txt"));
             tmpFile.setAutoRemove(false);
             tmpFile.open();
             tmpFile.write(text.toUtf8());
@@ -137,11 +176,12 @@ bool SharePlugin::receivePackage(const NetworkPackage& np)
     return true;
 }
 
-void SharePlugin::finished(KJob* job)
+void SharePlugin::finished(KJob* job, const qint64 dateModified)
 {
     FileTransferJob* ftjob = qobject_cast<FileTransferJob*>(job);
     if (ftjob && !job->error()) {
         Q_EMIT shareReceived(ftjob->destination().toString());
+        setDateModified(ftjob->destination(), dateModified);
         qCDebug(KDECONNECT_PLUGIN_SHARE) << "File transfer finished." << ftjob->destination();
     } else {
         qCDebug(KDECONNECT_PLUGIN_SHARE) << "File transfer failed." << (ftjob ? ftjob->destination() : QUrl());
@@ -153,22 +193,42 @@ void SharePlugin::openDestinationFolder()
     QDesktopServices::openUrl(destinationDir());
 }
 
-void SharePlugin::shareUrl(const QUrl& url)
+void SharePlugin::shareUrl(const QUrl& url, bool open)
 {
-    NetworkPackage package(PACKAGE_TYPE_SHARE_REQUEST);
-    if(url.isLocalFile()) {
-        QSharedPointer<QIODevice> ioFile(new QFile(url.toLocalFile()));
-        package.setPayload(ioFile, ioFile->size());
-        package.set<QString>(QStringLiteral("filename"), QUrl(url).fileName());
+    NetworkPacket packet(PACKET_TYPE_SHARE_REQUEST);
+    if (url.isLocalFile()) {
+        QSharedPointer<QFile> ioFile(new QFile(url.toLocalFile()));
+
+        if (!ioFile->exists()) {
+            Daemon::instance()->reportError(i18n("Could not share file"), i18n("%1 does not exist", url.toLocalFile()));
+            return;
+        } else {
+            packet.setPayload(ioFile, ioFile->size());
+            packet.set<QString>(QStringLiteral("filename"), QUrl(url).fileName());
+            packet.set<bool>(QStringLiteral("open"), open);
+        }
     } else {
-        package.set<QString>(QStringLiteral("url"), url.toString());
+        packet.set<QString>(QStringLiteral("url"), url.toString());
     }
-    sendPackage(package);
+    sendPacket(packet);
+}
+
+void SharePlugin::shareUrls(const QStringList& urls) {
+    for(const QString& url : urls) {
+        shareUrl(QUrl(url), false);
+    }
+}
+
+void SharePlugin::shareText(const QString& text)
+{
+    NetworkPacket packet(PACKET_TYPE_SHARE_REQUEST);
+    packet.set<QString>(QStringLiteral("text"), text);
+    sendPacket(packet);
 }
 
 QString SharePlugin::dbusPath() const
 {
-    return "/modules/kdeconnect/devices/" + device()->id() + "/share";
+    return QStringLiteral("/modules/kdeconnect/devices/") + device()->id() + QStringLiteral("/share");
 }
 
 #include "shareplugin.moc"
